@@ -55,11 +55,10 @@
 #' @param drop Provide a vector of character strings to specify the names of any parameters that you do not want MCMC samples for. Dropping parameters in this way can improve sampling speed and reduce memory usage. The following parameter vectors can potentially be dropped from GLM models:
 #' \describe{
 #' \item{'fitted'}{The N-length vector of fitted values}
-#' \item{'log_lik'}{The N-length vector of pointwise log-likelihoods, which is used to calculate WAIC.}
 #' \item{'alpha_re'}{Vector of 'random effects'/varying intercepts.}
 #' \item{'x_true'}{N-length vector of 'latent'/modeled covariate values created for measurement error (ME) models.}
 #' }
-#' Using `drop = c('fitted', 'log_lik', 'alpha_re', 'x_true')` is equivalent to `slim = TRUE`. If `slim = TRUE`, then `drop` will be ignored.
+#' Using `drop = c('fitted', 'alpha_re', 'x_true')` is equivalent to `slim = TRUE`. If `slim = TRUE`, then `drop` will be ignored.
 #' @param control A named list of parameters to control the sampler's behavior. See \link[rstan]{stan} for details. 
 #' @param ... Other arguments passed to \link[rstan]{sampling}.
 #' @param quiet Controls (most) automatic printing to the console. By default, any prior distributions that have not been assigned by the user are printed to the console. If `quiet = TRUE`, these will not be printed. Using `quiet = TRUE` will also force `refresh = 0`.
@@ -352,7 +351,7 @@ stan_glm <- function(formula,
     W_w = as.array(W.list$w),
     W_v = as.array(W.list$v),
     W_u = as.array(W.list$u),
-    dw_nonzero = length(W.list$w),
+    nW_w = length(W.list$w),
     dwx = dwx,
     wx_idx = wx_idx
     )
@@ -368,12 +367,14 @@ stan_glm <- function(formula,
                                offset = offset[y_index_list$y_obs_idx])    
     standata <- append_priors(standata, priors_made)
     ## EMPTY PLACEHOLDERS
-    standata <- c(standata, empty_icar_data(n), empty_esf_data(n), empty_car_data(), empty_sar_data(n))
+    empty_parts <- c(empty_icar_data(n), empty_esf_data(n), empty_car_data(), empty_sar_data(n))
+    empty_parts <- empty_parts[ which(!names(empty_parts) %in% names(standata)) ]
+    standata <- c(standata, empty_parts)
     ## ME MODEL -------------  
     me.list <- make_me_data(ME, xraw)
     standata <- c(standata, me.list)  
     ## PARAMETERS TO KEEP -------------               
-    pars <- c(pars, 'intercept', 'fitted', 'log_lik')
+    pars <- c(pars, 'intercept', 'fitted')
     if (!intercept_only) pars <- c(pars, 'beta')
     if (dwx) pars <- c(pars, 'gamma')
     if (family$family %in% c("gaussian", "student_t")) pars <- c(pars, 'sigma')
@@ -387,7 +388,7 @@ stan_glm <- function(formula,
             pars <- c(pars, "nu_x_true")
         }
     }
-    if (slim == TRUE) drop <- c('fitted', 'log_lik', 'alpha_re', 'x_true')
+    if (slim == TRUE) drop <- c('fitted', 'alpha_re', 'x_true')
     pars <- drop_params(pars = pars, drop_list = drop)
     priors_made_slim <- priors_made[which(names(priors_made) %in% pars)]
     if (me.list$has_me) priors_made_slim$ME_model <- ME$prior        
@@ -406,7 +407,7 @@ stan_glm <- function(formula,
     }    
     samples <- rstan::sampling(stanmodels$foundation, data = standata, iter = iter, chains = chains, refresh = refresh, pars = xparsx, control = control, init = inits, ...) 
     ## OUTPUT -------------    
-    out <- clean_results(samples, pars, is_student, has_re, Wx, xraw, me.list$x_me_idx)
+    out <- clean_results(samples, pars, is_student, has_re, Wx, xraw, me.list$x_me_idx)        
     out$data <- data.frame(as.matrix(ModData))
     out$family <- family
     out$formula <- formula
@@ -421,12 +422,22 @@ stan_glm <- function(formula,
     } else {
         out$spatial <- data.frame(par = "none", method = "none")
     }
-    if (!missing(C) && any(pars == 'fitted')) {
-        out$C <- as(C, "sparseMatrix")        
-        R <- resid(out, summary = FALSE)
-        out$diagnostic["Residual_MC"] <- mean( apply(R, 1, mc, w = C, warn = FALSE, na.rm = TRUE) )
-    }
+    if (!missing(C)) out$C <- as(C, "sparseMatrix")    
     out$N <- length( y_index_list$y_obs_idx )
+    out$missing <- y_index_list
+
+    out$diagnostic <- list()    
+    if (!missing(C) && any(pars == 'fitted')) {
+        C <- as(C, "sparseMatrix")        
+        R <- resid(out, summary = FALSE)
+        rmc <- mean( apply(R, 1, mc, w = C, warn = FALSE, na.rm = TRUE) )
+        out$diagnostic$Residual_MC <- rmc
+    }    
+    if (any(pars == 'fitted')) {
+        out$diagnostic$WAIC <- as.numeric(waic(out)[1])
+    }                                                        
+
     return(out)
+    
 }
 
